/****************************************************************************
** Meta object code from reading C++ file 'qcanvas2ditem_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcanvaspainter/src/canvas2d/qcanvas2ditem_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcanvas2ditem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QCanvas2DItemE_t {};
} // unnamed namespace

template <> constexpr inline auto QCanvas2DItem::qt_create_metaobjectdata<qt_meta_tag_ZN13QCanvas2DItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCanvas2DItem",
        "QML.Element",
        "Canvas2D",
        "paint",
        "",
        "painted",
        "availableChanged",
        "contextTypeChanged",
        "contextChanged",
        "imageLoaded",
        "loadImage",
        "QUrl",
        "url",
        "QSizeF",
        "sourceSize",
        "unloadImage",
        "isImageLoaded",
        "isImageLoading",
        "isImageError",
        "sceneGraphInitialized",
        "checkAnimationCallbacks",
        "invalidateSceneGraph",
        "schedulePolish",
        "getContext",
        "QQmlV4FunctionPtr",
        "args",
        "requestAnimationFrame",
        "cancelRequestAnimationFrame",
        "requestPaint",
        "markDirty",
        "delayedCreate",
        "available",
        "contextType",
        "context",
        "QJSValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'paint'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'painted'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'availableChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contextTypeChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contextChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imageLoaded'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadImage'
        QtMocHelpers::SlotData<void(const QUrl &, QSizeF)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'loadImage'
        QtMocHelpers::SlotData<void(const QUrl &)>(10, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'unloadImage'
        QtMocHelpers::SlotData<void(const QUrl &)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'isImageLoaded'
        QtMocHelpers::SlotData<bool(const QUrl &) const>(16, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'isImageLoading'
        QtMocHelpers::SlotData<bool(const QUrl &) const>(17, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'isImageError'
        QtMocHelpers::SlotData<bool(const QUrl &) const>(18, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'sceneGraphInitialized'
        QtMocHelpers::SlotData<void()>(19, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkAnimationCallbacks'
        QtMocHelpers::SlotData<void()>(20, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'invalidateSceneGraph'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'schedulePolish'
        QtMocHelpers::SlotData<void()>(22, 4, QMC::AccessPrivate, QMetaType::Void),
        // Method 'getContext'
        QtMocHelpers::MethodData<void(QQmlV4FunctionPtr)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'requestAnimationFrame'
        QtMocHelpers::MethodData<void(QQmlV4FunctionPtr)>(26, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'cancelRequestAnimationFrame'
        QtMocHelpers::MethodData<void(QQmlV4FunctionPtr)>(27, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'requestPaint'
        QtMocHelpers::MethodData<void()>(28, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'markDirty'
        QtMocHelpers::MethodData<void()>(29, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'delayedCreate'
        QtMocHelpers::MethodData<void()>(30, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'available'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 2),
        // property 'contextType'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'context'
        QtMocHelpers::PropertyData<QJSValue>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QCanvas2DItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCanvas2DItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QCanvasPainterItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QCanvas2DItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QCanvas2DItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QCanvas2DItemE_t>.metaTypes,
    nullptr
} };

void QCanvas2DItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QCanvas2DItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->paint(); break;
        case 1: _t->painted(); break;
        case 2: _t->availableChanged(); break;
        case 3: _t->contextTypeChanged(); break;
        case 4: _t->contextChanged(); break;
        case 5: _t->imageLoaded(); break;
        case 6: _t->loadImage((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[2]))); break;
        case 7: _t->loadImage((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->unloadImage((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 9: { bool _r = _t->isImageLoaded((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->isImageLoading((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->isImageError((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->sceneGraphInitialized(); break;
        case 13: _t->checkAnimationCallbacks(); break;
        case 14: _t->invalidateSceneGraph(); break;
        case 15: _t->schedulePolish(); break;
        case 16: _t->getContext((*reinterpret_cast<std::add_pointer_t<QQmlV4FunctionPtr>>(_a[1]))); break;
        case 17: _t->requestAnimationFrame((*reinterpret_cast<std::add_pointer_t<QQmlV4FunctionPtr>>(_a[1]))); break;
        case 18: _t->cancelRequestAnimationFrame((*reinterpret_cast<std::add_pointer_t<QQmlV4FunctionPtr>>(_a[1]))); break;
        case 19: _t->requestPaint(); break;
        case 20: _t->markDirty(); break;
        case 21: _t->delayedCreate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QCanvas2DItem::*)()>(_a, &QCanvas2DItem::paint, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCanvas2DItem::*)()>(_a, &QCanvas2DItem::painted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCanvas2DItem::*)()>(_a, &QCanvas2DItem::availableChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCanvas2DItem::*)()>(_a, &QCanvas2DItem::contextTypeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCanvas2DItem::*)()>(_a, &QCanvas2DItem::contextChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCanvas2DItem::*)()>(_a, &QCanvas2DItem::imageLoaded, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isAvailable(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->contextType(); break;
        case 2: *reinterpret_cast<QJSValue*>(_v) = _t->context(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setContextType(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QCanvas2DItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QCanvas2DItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QCanvas2DItemE_t>.strings))
        return static_cast<void*>(this);
    return QCanvasPainterItem::qt_metacast(_clname);
}

int QCanvas2DItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCanvasPainterItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QCanvas2DItem::paint()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QCanvas2DItem::painted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QCanvas2DItem::availableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QCanvas2DItem::contextTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QCanvas2DItem::contextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QCanvas2DItem::imageLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
