/****************************************************************************
** Meta object code from reading C++ file 'qcanvas2dpathvaluetype_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcanvaspainter/src/canvas2d/qcanvas2dpathvaluetype_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcanvas2dpathvaluetype_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22QCanvas2DPathValueTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto QCanvas2DPathValueType::qt_create_metaobjectdata<qt_meta_tag_ZN22QCanvas2DPathValueTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCanvas2DPathValueType",
        "QML.AddedInVersion",
        "1548",
        "QML.Foreign",
        "QCanvasPath",
        "QML.Element",
        "path2d",
        "QML.Extended",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "closePath",
        "",
        "moveTo",
        "x",
        "y",
        "lineTo",
        "bezierCurveTo",
        "cp1x",
        "cp1y",
        "cp2x",
        "cp2y",
        "quadraticCurveTo",
        "cpx",
        "cpy",
        "arcTo",
        "x1",
        "y1",
        "x2",
        "y2",
        "radius",
        "arc",
        "startAngle",
        "endAngle",
        "counterClockWise",
        "rect",
        "width",
        "height",
        "roundRect",
        "radiusTopLeft",
        "radiusTopRight",
        "radiusBottomRight",
        "radiusBottomLeft",
        "ellipse",
        "radiusX",
        "radiusY",
        "ellipseRect",
        "circle",
        "setPathWinding",
        "winding",
        "beginSolidSubPath",
        "beginHoleSubPath",
        "addPath",
        "path",
        "QTransform",
        "transform",
        "qsizetype",
        "start",
        "count",
        "svgPath",
        "isEmpty",
        "clear"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'closePath'
        QtMocHelpers::MethodData<void()>(12, 13, QMC::AccessPublic, QMetaType::Void),
        // Method 'moveTo'
        QtMocHelpers::MethodData<void(float, float)>(14, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 },
        }}),
        // Method 'lineTo'
        QtMocHelpers::MethodData<void(float, float)>(17, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 },
        }}),
        // Method 'bezierCurveTo'
        QtMocHelpers::MethodData<void(float, float, float, float, float, float)>(18, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 }, { QMetaType::Float, 20 }, { QMetaType::Float, 21 }, { QMetaType::Float, 22 },
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 },
        }}),
        // Method 'quadraticCurveTo'
        QtMocHelpers::MethodData<void(float, float, float, float)>(23, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 24 }, { QMetaType::Float, 25 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 },
        }}),
        // Method 'arcTo'
        QtMocHelpers::MethodData<void(float, float, float, float, float)>(26, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 27 }, { QMetaType::Float, 28 }, { QMetaType::Float, 29 }, { QMetaType::Float, 30 },
            { QMetaType::Float, 31 },
        }}),
        // Method 'arc'
        QtMocHelpers::MethodData<void(float, float, float, float, float, bool)>(32, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 31 }, { QMetaType::Float, 33 },
            { QMetaType::Float, 34 }, { QMetaType::Bool, 35 },
        }}),
        // Method 'arc'
        QtMocHelpers::MethodData<void(float, float, float, float, float)>(32, 13, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 31 }, { QMetaType::Float, 33 },
            { QMetaType::Float, 34 },
        }}),
        // Method 'rect'
        QtMocHelpers::MethodData<void(float, float, float, float)>(36, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 37 }, { QMetaType::Float, 38 },
        }}),
        // Method 'roundRect'
        QtMocHelpers::MethodData<void(float, float, float, float, float)>(39, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 37 }, { QMetaType::Float, 38 },
            { QMetaType::Float, 31 },
        }}),
        // Method 'roundRect'
        QtMocHelpers::MethodData<void(float, float, float, float, float, float, float, float)>(39, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 37 }, { QMetaType::Float, 38 },
            { QMetaType::Float, 40 }, { QMetaType::Float, 41 }, { QMetaType::Float, 42 }, { QMetaType::Float, 43 },
        }}),
        // Method 'ellipse'
        QtMocHelpers::MethodData<void(float, float, float, float)>(44, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 45 }, { QMetaType::Float, 46 },
        }}),
        // Method 'ellipseRect'
        QtMocHelpers::MethodData<void(float, float, float, float)>(47, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 37 }, { QMetaType::Float, 38 },
        }}),
        // Method 'circle'
        QtMocHelpers::MethodData<void(float, float, float)>(48, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 31 },
        }}),
        // Method 'setPathWinding'
        QtMocHelpers::MethodData<void(const QString &)>(49, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 50 },
        }}),
        // Method 'beginSolidSubPath'
        QtMocHelpers::MethodData<void()>(51, 13, QMC::AccessPublic, QMetaType::Void),
        // Method 'beginHoleSubPath'
        QtMocHelpers::MethodData<void()>(52, 13, QMC::AccessPublic, QMetaType::Void),
        // Method 'addPath'
        QtMocHelpers::MethodData<void(const QCanvasPath &, const QTransform &)>(53, 13, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 54 }, { 0x80000000 | 55, 56 },
        }}),
        // Method 'addPath'
        QtMocHelpers::MethodData<void(const QCanvasPath &)>(53, 13, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 4, 54 },
        }}),
        // Method 'addPath'
        QtMocHelpers::MethodData<void(const QCanvasPath &, qsizetype, qsizetype, const QTransform &)>(53, 13, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 54 }, { 0x80000000 | 57, 58 }, { 0x80000000 | 57, 59 }, { 0x80000000 | 55, 56 },
        }}),
        // Method 'addPath'
        QtMocHelpers::MethodData<void(const QCanvasPath &, qsizetype, qsizetype)>(53, 13, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 4, 54 }, { 0x80000000 | 57, 58 }, { 0x80000000 | 57, 59 },
        }}),
        // Method 'addPath'
        QtMocHelpers::MethodData<void(const QString &, const QTransform &)>(53, 13, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 60 }, { 0x80000000 | 55, 56 },
        }}),
        // Method 'addPath'
        QtMocHelpers::MethodData<void(const QString &)>(53, 13, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 60 },
        }}),
        // Method 'isEmpty'
        QtMocHelpers::MethodData<bool() const>(61, 13, QMC::AccessPublic, QMetaType::Bool),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(62, 13, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor()>(13, QMC::AccessPublic),
    };
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    0 },
            {    8,    9 },
            {   10,   11 },
    });
    return QtMocHelpers::metaObjectData<QCanvas2DPathValueType, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCanvas2DPathValueType::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QCanvasPath>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QCanvas2DPathValueTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QCanvas2DPathValueTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22QCanvas2DPathValueTypeE_t>.metaTypes,
    nullptr
} };

void QCanvas2DPathValueType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QCanvas2DPathValueType *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { QCanvas2DPathValueType *_r = new QCanvas2DPathValueType();
            if (_a[0]) *reinterpret_cast<void**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) QCanvas2DPathValueType(); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->closePath(); break;
        case 1: _t->moveTo((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 2: _t->lineTo((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 3: _t->bezierCurveTo((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6]))); break;
        case 4: _t->quadraticCurveTo((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4]))); break;
        case 5: _t->arcTo((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5]))); break;
        case 6: _t->arc((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 7: _t->arc((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5]))); break;
        case 8: _t->rect((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4]))); break;
        case 9: _t->roundRect((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5]))); break;
        case 10: _t->roundRect((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[8]))); break;
        case 11: _t->ellipse((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4]))); break;
        case 12: _t->ellipseRect((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4]))); break;
        case 13: _t->circle((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 14: _t->setPathWinding((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->beginSolidSubPath(); break;
        case 16: _t->beginHoleSubPath(); break;
        case 17: _t->addPath((*reinterpret_cast<std::add_pointer_t<QCanvasPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTransform>>(_a[2]))); break;
        case 18: _t->addPath((*reinterpret_cast<std::add_pointer_t<QCanvasPath>>(_a[1]))); break;
        case 19: _t->addPath((*reinterpret_cast<std::add_pointer_t<QCanvasPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QTransform>>(_a[4]))); break;
        case 20: _t->addPath((*reinterpret_cast<std::add_pointer_t<QCanvasPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[3]))); break;
        case 21: _t->addPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTransform>>(_a[2]))); break;
        case 22: _t->addPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: { bool _r = _t->isEmpty();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->clear(); break;
        default: ;
        }
    }
}
QT_WARNING_POP
