/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/qcanvas2dbrushvaluetype_p.h>)
#  include <private/qcanvas2dbrushvaluetype_p.h>
#endif
#if __has_include(<private/qcanvas2ditem_p.h>)
#  include <private/qcanvas2ditem_p.h>
#endif
#if __has_include(<private/qcanvas2dpathvaluetype_p.h>)
#  include <private/qcanvas2dpathvaluetype_p.h>
#endif
#if __has_include(<private/qcanvas2dtransformvaluetype_p.h>)
#  include <private/qcanvas2dtransformvaluetype_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtCanvas2D()
{
    qmlRegisterModule("QtCanvas2D", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<QCanvas2DBoxGradientValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DBoxGradientValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DBoxShadowValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DBoxShadowValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DConicalGradientValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DConicalGradientValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DGridPatternValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DGridPatternValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DImagePatternValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DImagePatternValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DItem>("QtCanvas2D", 6);
    qmlRegisterAnonymousTypesAndRevisions<QQuickItem>("QtCanvas2D", 6);
    qmlRegisterTypesAndRevisions<QCanvas2DLinearGradientValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DLinearGradientValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DPathValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DPathValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DRadialGradientValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DRadialGradientValueType *>().id();
    qmlRegisterTypesAndRevisions<QCanvas2DTransformValueType>("QtCanvas2D", 6);
    QMetaType::fromType<QCanvas2DTransformValueType *>().id();
    QMetaType::fromType<QCanvasPainterItem *>().id();
    QMetaType::fromType<QQuickRhiItem *>().id();
    qmlRegisterEnum<QQuickRhiItem::TextureFormat>("QQuickRhiItem::TextureFormat");
    QT_WARNING_POP
    qmlRegisterModule("QtCanvas2D", 6, 12);
}

static const QQmlModuleRegistration qtCanvas2DRegistration("QtCanvas2D", qml_register_types_QtCanvas2D);
